"use strict";

exports.__esModule = true;

var _gxbjs = require("gxbjs");

var _superagent = require("superagent");

var _superagent2 = _interopRequireDefault(_superagent);

var _camelCase = require("lodash/camelCase");

var _camelCase2 = _interopRequireDefault(_camelCase);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var DESBlacklistGatewayClient = function () {
    // DES Base URL

    // Private Key
    function DESBlacklistGatewayClient(privateKey, account_id) {
        var baseURL = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : "https://survey.gxb.io";

        _classCallCheck(this, DESBlacklistGatewayClient);

        this.privateKey = null;
        this.account_id = "";
        this.baseURL = "";

        this.privateKey = _gxbjs.PrivateKey.fromWif(privateKey);
        this.account_id = account_id;
        this.baseURL = baseURL;
    }

    /**
     * get question token
     * @param requestId
     * @param redirectUrl
     * @param loanInfoList
     * @returns {Promise<any>}
     */
    // Account ID


    DESBlacklistGatewayClient.prototype.getQuestionToken = function getQuestionToken(requestId, redirectUrl) {
        var _this = this;

        var loanInfoList = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];

        return new Promise(function (resolve, reject) {
            var loanInfoStr = "";
            var timestamp = new Date().getTime() + 60000;
            loanInfoList.forEach(function (loanInfo) {
                loanInfoStr += loanInfo.platform + "|" + loanInfo.app_id + "|" + loanInfo.loan_type + "|" + loanInfo.loan_status + "|" + loanInfo.loan_amount + "|" + loanInfo.contract_date + "|" + loanInfo.repay_status + "|" + loanInfo.arrears.toFixed(2) + "|";
            });
            var tokenString = requestId + "|" + timestamp + "|" + redirectUrl + "|" + loanInfoStr;
            var signature = _gxbjs.Signature.sign(tokenString, _this.privateKey);

            var body = {
                requestId: requestId,
                redirectUrl: redirectUrl,
                timestamp: timestamp,
                signature: signature.toHex(),
                loanInfos: loanInfoList.map(function (info) {
                    var result = {};
                    for (var key in info) {
                        result[(0, _camelCase2.default)(key)] = info[key];
                    }
                    return result;
                })
            };

            console.log(JSON.stringify(body));
            _superagent2.default.post(_this.baseURL + "/blacklist/token").send(body).end(function (err, resp) {
                if (err) {
                    reject(err);
                } else {
                    resolve(resp.body);
                }
            });
        });
    };

    /**
     * get question report
     * @param token
     * @returns {Promise<any>}
     */


    DESBlacklistGatewayClient.prototype.getQuestionReport = function getQuestionReport(token) {
        var _this2 = this;

        return new Promise(function (resolve, reject) {
            var timestamp = new Date().getTime() + 60000;
            var tokenString = token + "|" + timestamp;
            var signature = _gxbjs.Signature.sign(tokenString, _this2.privateKey);
            var body = {
                token: token,
                timestamp: timestamp,
                signature: signature.toHex()
            };
            console.log(body);
            _superagent2.default.post(_this2.baseURL + "/blacklist/question/report").send(body).end(function (err, resp) {
                if (err) {
                    reject(err);
                } else {
                    resolve(resp.body);
                }
            });
        });
    };

    return DESBlacklistGatewayClient;
}();

exports.default = DESBlacklistGatewayClient;
module.exports = exports["default"];