"use strict";

exports.__esModule = true;

var _gxbjs = require("gxbjs");

var _superagent = require("superagent");

var _superagent2 = _interopRequireDefault(_superagent);

var _util = require("../common/util");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var DESDatasourceClient = function () {
    // DES Base URL

    // Private Key
    function DESDatasourceClient(privateKey, account_id) {
        var baseURL = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : "https://des.gxchain.cn";

        _classCallCheck(this, DESDatasourceClient);

        this.privateKey = null;
        this.account_id = "";
        this.baseURL = "";

        this.privateKey = _gxbjs.PrivateKey.fromWif(privateKey);
        this.account_id = account_id;
        this.baseURL = baseURL;
    }

    /**
     * heart beat to des server
     * @param products
     * @returns {Promise<any>}
     */
    // Account ID


    DESDatasourceClient.prototype.heartbeat = function heartbeat(products) {
        var _this = this;

        return new Promise(function (resolve, reject) {
            var timestamp = new Date().getTime() + 3000;
            var params = {
                account: _this.account_id,
                products: products,
                timestamp: timestamp,
                signature: _gxbjs.Signature.sign(_this.account_id + "|" + JSON.stringify(products) + "|" + timestamp).toHex()
            };
            _superagent2.default.post(_this.baseURL + "/api/datasource/heartbeat").send(params).end(function (err, resp) {
                if (err) {
                    reject(err);
                } else {
                    resolve(resp.body);
                }
            });
        });
    };

    /**
     * encrypt data with shared secret
     * @param params
     * @param publicKey
     * @returns {{data: string, nonce: *}}
     */


    DESDatasourceClient.prototype.encrypt = function encrypt(params, publicKey) {
        var nonce = _gxbjs.TransactionHelper.unique_nonce_uint64();
        return {
            data: (0, _util.encrypt_params)(params, this.privateKey, publicKey, nonce),
            nonce: nonce
        };
    };

    /**
     * decrypt message with shared secret
     * @param message
     * @param nonce
     * @param publicKey
     */


    DESDatasourceClient.prototype.decrypt = function decrypt(message, nonce, publicKey) {
        return (0, _util.decrypt_msg)(message, this.privateKey, publicKey, nonce);
    };

    return DESDatasourceClient;
}();

exports.default = DESDatasourceClient;
module.exports = exports["default"];