"use strict";

exports.__esModule = true;

var _superagent = require("superagent");

var _superagent2 = _interopRequireDefault(_superagent);

var _validator = require("../common/validator");

var _validator2 = _interopRequireDefault(_validator);

var _gxbjs = require("gxbjs");

var _util = require("../common/util");

var _cryptoJs = require("crypto-js");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var DEFAULT_TIMEOUT = 30 * 60 * 1000;

var DESMerchantClient = function () {
    // Account ID
    function DESMerchantClient(privateKey, account_id) {
        var baseURL = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : "https://des.gxchain.cn";

        _classCallCheck(this, DESMerchantClient);

        this.privateKey = null;
        this.account_id = "";
        this.baseURL = "";

        this.privateKey = _gxbjs.PrivateKey.fromWif(privateKey);
        this.account_id = account_id;
        this.baseURL = baseURL;
    }

    /**
     * fetch product info by product id
     * @param productId
     * @returns {Promise<any>}
     */
    // DES Base URL
    // Private Key


    DESMerchantClient.prototype.getProduct = function getProduct(productId) {
        var _this = this;

        return new Promise(function (resolve, reject) {
            _superagent2.default.get(_this.baseURL + "/api/product/" + productId).end(function (err, resp) {
                if (err) {
                    reject(err);
                } else {
                    resolve(resp.body);
                }
            });
        });
    };

    /**
     * create data exchange request
     * @param productId
     * @param params
     * @returns {Promise<any>}
     */


    DESMerchantClient.prototype.createDataExchangeRequest = function createDataExchangeRequest(productId, params) {
        var _this2 = this;

        return new Promise(function (resolve, reject) {
            _this2.getProduct(productId).then(function (prod) {
                try {
                    var reqBody = [];
                    var filteredParams = _validator2.default.validate(params, prod.product.input);
                    var bodyParam = {
                        params: filteredParams,
                        timestamp: new Date().getTime()
                    };
                    var expiration = Math.floor((new Date().getTime() + DEFAULT_TIMEOUT) / 1000) * 1000;
                    prod.onlineDatasources.forEach(function (datasource) {
                        DESMerchantClient.publicKeyMap[datasource.accountId] = datasource.publicKey;
                        var operation = {
                            from: _this2.account_id,
                            to: datasource.accountId,
                            proxy_account: prod.des.accountId,
                            percentage: prod.des.percent,
                            amount: {
                                amount: prod.product.price.amount,
                                asset_id: prod.product.price.assetId
                            },
                            expiration: new Date(expiration),
                            // memo:'5c37400e6e71b9faec6974e7fc219e72',
                            memo: (0, _cryptoJs.MD5)(JSON.stringify(bodyParam)).toString(),
                            signatures: []
                        };
                        var signature = _gxbjs.Signature.signBuffer(_gxbjs.ops.signed_proxy_transfer_params.toBuffer(operation), _this2.privateKey);
                        var nonce = _gxbjs.TransactionHelper.unique_nonce_uint64();
                        reqBody.push({
                            params: (0, _util.encrypt_params)(bodyParam, _this2.privateKey, datasource.publicKey, nonce),
                            nonce: nonce,
                            requestParams: {
                                from: operation.from,
                                to: operation.to,
                                proxyAccount: operation.proxy_account,
                                percent: operation.percentage,
                                amount: {
                                    amount: operation.amount.amount,
                                    assetId: operation.amount.asset_id
                                },
                                expiration: Math.floor(expiration / 1000),
                                memo: operation.memo,
                                signatures: [signature.toHex()]
                            }
                        });
                    });
                    _superagent2.default.post(_this2.baseURL + "/api/request/create/" + productId).send(reqBody).end(function (err, resp) {
                        if (err) {
                            reject(resp.body);
                        } else {
                            resolve(resp.body);
                        }
                    });
                } catch (ex) {
                    reject(ex);
                }
            });
        });
    };

    /**
     * fetch result by request id
     * @param requestId
     * @param timeout
     * @returns {Promise<any>}
     */


    DESMerchantClient.prototype.getResult = function getResult(requestId) {
        var _this4 = this;

        var timeout = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 8000;

        var start = new Date();
        var latestResult = null;
        return new Promise(function (resolve) {
            function innerFetch() {
                var _this3 = this;

                _superagent2.default.get(this.baseURL + "/api/request/" + requestId).end(function (err, resp) {
                    if (!err) {
                        latestResult = resp.body;
                        if (latestResult && latestResult.status !== "IN_PROGRESS") {
                            resolve(_this3.decryptResult(latestResult));
                        } else {
                            if (new Date() - start < timeout) {
                                setTimeout(function () {
                                    innerFetch.call(_this3);
                                }, 60);
                            } else {
                                resolve(_this3.decryptResult(latestResult));
                            }
                        }
                    } else {
                        if (new Date() - start < timeout) {
                            setTimeout(function () {
                                innerFetch.call(_this3);
                            }, 60);
                        } else {
                            resolve(_this3.decryptResult(latestResult));
                        }
                    }
                });
            }

            innerFetch.call(_this4);
        });
    };

    /**
     * decrypt result before it returned
     * @param result
     * @returns {*}
     */


    DESMerchantClient.prototype.decryptResult = function decryptResult(result) {
        var _this5 = this;

        if (result && result.datasources) {
            result.datasources = result.datasources.map(function (item) {
                var newItem = item;
                if (newItem.status === "SUCCESS") {
                    newItem.data = JSON.parse((0, _util.decrypt_msg)(newItem.data, _this5.privateKey, newItem.datasourcePublicKey, newItem.nonce));
                }
                return newItem;
            });
        }
        return result;
    };

    return DESMerchantClient;
}();

DESMerchantClient.publicKeyMap = {};
exports.default = DESMerchantClient;
module.exports = exports["default"];