function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

import { PrivateKey, Signature, TransactionHelper } from "gxbjs";
import superagent from "superagent";
import { decrypt_msg, encrypt_params } from "../common/util";

var DESDatasourceClient = function () {
    // DES Base URL

    // Private Key
    function DESDatasourceClient(privateKey, account_id) {
        var baseURL = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : "https://des.gxchain.cn";

        _classCallCheck(this, DESDatasourceClient);

        this.privateKey = null;
        this.account_id = "";
        this.baseURL = "";

        this.privateKey = PrivateKey.fromWif(privateKey);
        this.account_id = account_id;
        this.baseURL = baseURL;
    }

    /**
     * heart beat to des server
     * @param products
     * @returns {Promise<any>}
     */
    // Account ID


    DESDatasourceClient.prototype.heartbeat = function heartbeat(products) {
        var _this = this;

        return new Promise(function (resolve, reject) {
            var timestamp = new Date().getTime() + 3000;
            var params = {
                account: _this.account_id,
                products: products,
                timestamp: timestamp,
                signature: Signature.sign(_this.account_id + "|" + JSON.stringify(products) + "|" + timestamp).toHex()
            };
            superagent.post(_this.baseURL + "/api/datasource/heartbeat").send(params).end(function (err, resp) {
                if (err) {
                    reject(err);
                } else {
                    resolve(resp.body);
                }
            });
        });
    };

    /**
     * encrypt data with shared secret
     * @param params
     * @param publicKey
     * @returns {{data: string, nonce: *}}
     */


    DESDatasourceClient.prototype.encrypt = function encrypt(params, publicKey) {
        var nonce = TransactionHelper.unique_nonce_uint64();
        return {
            data: encrypt_params(params, this.privateKey, publicKey, nonce),
            nonce: nonce
        };
    };

    /**
     * decrypt message with shared secret
     * @param message
     * @param nonce
     * @param publicKey
     */


    DESDatasourceClient.prototype.decrypt = function decrypt(message, nonce, publicKey) {
        return decrypt_msg(message, this.privateKey, publicKey, nonce);
    };

    return DESDatasourceClient;
}();

export default DESDatasourceClient;